import clarabel;
import numpy as np;
from scipy import sparse;

# Exponential cone example
# max  x
# s.t. y * exp(x / y) <= z
#      y == 1, z == exp(5)

# Define problem data
P = sparse.csc_matrix((3,3))

q = np.array([-1.,0.,0.])

A = sparse.csc_matrix( \
    [[ -1., 0., 0.],        
     [  0.,-1., 0],        
     [  0., 0.,-1.],        
     [  0., 1., 0.],       
     [  0., 0., 1.]]);     

b = np.array([0.,0.,0.,1.,np.exp(5.)])

cones = [clarabel.ExponentialConeT(), clarabel.ZeroConeT(2)]
settings = clarabel.DefaultSettings()

solver = clarabel.DefaultSolver(P,q,A,b,cones,settings)
solver.solve()



import clarabel
import numpy as np
from scipy import sparse

# Define problem data
n = 20000
P = sparse.identity(n).tocsc()

A1 = sparse.identity(n).tocsc()
A2 = -sparse.identity(n).tocsc()
A = sparse.vstack([A1, A2]).tocsc()

q = np.ones(n)
b = np.ones(2*n)

cones = [clarabel.NonnegativeConeT(b.size)]
settings = clarabel.DefaultSettings()

solver = clarabel.DefaultSolver(P, q, A, b, cones, settings)
solution = solver.solve()



import clarabel;
import numpy as np;
from scipy import sparse;

# Define problem data
P = sparse.csc_matrix([[0., 0.], [0., 2.]])
P = P.tocsc()

q = np.array([0.,0.])

A = sparse.csc_matrix( \
    [[ 0.,  0.],
     [-2.,  0.],
     [ 0., -1.]])

b = np.array([1.,-2.,-2.])

cones = [clarabel.SecondOrderConeT(3)]
settings = clarabel.DefaultSettings()
settings.max_iter = 15
settings.verbose = True

solver = clarabel.DefaultSolver(P,q,A,b,cones,settings);
solution = solver.solve()
print(
    f"Solver terminated with status: {solution.status}, objective {solution.obj_val},\n"
    f"and solution: {dict(s=solution.s, x=solution.x, z=solution.z)}"
)



import clarabel
import numpy as np
from scipy import sparse

# Define problem data
P = sparse.csc_matrix([[6., 0.], [0., 4.]])
P = sparse.triu(P).tocsc()

q = np.array([-1., -4.])

A = sparse.csc_matrix(
    [[1., -2.],        # <-- LHS of equality constraint (lower bound)
     [1.,  0.],        # <-- LHS of inequality constraint (upper bound)
     [0.,  1.],        # <-- LHS of inequality constraint (upper bound)
     [-1.,  0.],       # <-- LHS of inequality constraint (lower bound)
     [0., -1.]])       # <-- LHS of inequality constraint (lower bound)

b = np.array([0., 1., 1., 1., 1.])

cones = [clarabel.ZeroConeT(1), clarabel.NonnegativeConeT(4)]
settings = clarabel.DefaultSettings()

solver = clarabel.DefaultSolver(P, q, A, b, cones, settings)
solution = solver.solve()
print(
    f"Solver terminated with solution"
    f"{dict(s=solution.s, x=solution.x, z=solution.z)}"
)



import clarabel
import numpy as np
from scipy import sparse


n    = 3
nvec = int(n*(n+1)/2) 

# Define problem data
P = sparse.csc_matrix((nvec,nvec))
P = P.tocsc()

q = np.array([1.,0.,1.,0.,0.,1.])
sqrt2 = np.sqrt(2.)

A = sparse.csc_matrix( \
    [[-1., 0., 0., 0., 0., 0.],
     [0., -sqrt2, 0., 0., 0., 0.],
     [0., 0., -1., 0., 0., 0.],
     [0., 0., 0., -sqrt2, 0., 0.],
     [0., 0., 0., 0., -sqrt2, 0.],
     [0., 0., 0., 0., 0., -1.],
     [1., 4., 3., 8., 10., 6.],]);    

b = np.append(np.zeros(nvec),1.)

cones = [clarabel.PSDTriangleConeT(n),    
         clarabel.ZeroConeT(1)] 

settings = clarabel.DefaultSettings()

solver = clarabel.DefaultSolver(P,q,A,b,cones,settings)

solver.solve()
